@@ -21,8 +21,8 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
@@ -30,14 +30,12 @@
 #include "config.h"
 
 #define DOCTEST_CONFIG_IMPLEMENT_WITH_MAIN
-#include "doctest.hh"
-
 #include "byte_array.hh"
+#include "doctest/doctest.h"
 #include "lnav_config.hh"
+#include "lnav_util.hh"
 #include "relative_time.hh"
 #include "unique_path.hh"
-#include "logfile.hh"
-#include "log_format.hh"
 
 using namespace std;
 
@@ -61,7 +59,8 @@ TEST_CASE("overwritten-logfile") {
 }
 #endif
 
-TEST_CASE("byte_array") {
+TEST_CASE("byte_array")
+{
     using my_array_t = byte_array<8>;
 
     my_array_t ba1;
@@ -82,19 +81,26 @@ TEST_CASE("byte_array") {
     ba1.clear();
     CHECK(ba1.to_string() == "0000000000000000");
     CHECK(ba2.to_string() == "6162636431323334");
+
+    auto outbuf = auto_buffer::alloc(my_array_t::STRING_SIZE);
+    ba2.to_string(std::back_inserter(outbuf));
+    CHECK(std::string(outbuf.in(), outbuf.size()) == "6162636431323334");
 }
 
-TEST_CASE("ptime_fmt") {
-    const char *date_str = "2018-05-16 18:16:42";
+TEST_CASE("ptime_fmt")
+{
+    const char* date_str = "2018-05-16 18:16:42";
     struct exttm tm;
     off_t off = 0;
 
-    bool rc = ptime_fmt("%Y-%d-%m\t%H:%M:%S", &tm, date_str, off, strlen(date_str));
+    bool rc
+        = ptime_fmt("%Y-%d-%m\t%H:%M:%S", &tm, date_str, off, strlen(date_str));
     CHECK(!rc);
     CHECK(off == 8);
 }
 
-TEST_CASE("rgb_color from string") {
+TEST_CASE("rgb_color from string")
+{
     string name = "SkyBlue1";
     auto color = rgb_color::from_str(name).unwrap();
     CHECK(color.rc_r == 135);
@@ -102,14 +108,15 @@ TEST_CASE("rgb_color from string") {
     CHECK(color.rc_b == 255);
 }
 
-TEST_CASE("ptime_roundtrip") {
-    const char *fmts[] = {
+TEST_CASE("ptime_roundtrip")
+{
+    const char* fmts[] = {
         "%Y-%m-%d %l:%M:%S %p",
         "%Y-%m-%d %I:%M:%S %p",
     };
     time_t now = time(nullptr);
 
-    for (auto fmt : fmts) {
+    for (const auto* fmt : fmts) {
         for (time_t sec = now; sec < (now + (24 * 60 * 60)); sec++) {
             char ftime_result[128];
             char strftime_result[128];
@@ -118,40 +125,39 @@ TEST_CASE("ptime_roundtrip") {
             memset(&etm, 0, sizeof(etm));
             gmtime_r(&sec, &etm.et_tm);
             etm.et_flags = ETF_YEAR_SET | ETF_MONTH_SET | ETF_DAY_SET;
-            size_t ftime_size = ftime_fmt(ftime_result, sizeof(ftime_result),
-                                          fmt, etm);
-            size_t strftime_size = strftime(strftime_result,
-                                            sizeof(strftime_result), fmt,
-                                            &etm.et_tm);
+            size_t ftime_size
+                = ftime_fmt(ftime_result, sizeof(ftime_result), fmt, etm);
+            size_t strftime_size = strftime(
+                strftime_result, sizeof(strftime_result), fmt, &etm.et_tm);
 
-                CHECK(string(ftime_result, ftime_size) ==
-                      string(strftime_result, strftime_size));
+            CHECK(string(ftime_result, ftime_size)
+                  == string(strftime_result, strftime_size));
 
             struct exttm etm2;
             off_t off = 0;
 
             memset(&etm2, 0, sizeof(etm2));
             bool rc = ptime_fmt(fmt, &etm2, ftime_result, off, ftime_size);
-                CHECK(rc);
-                CHECK(sec == tm2sec(&etm2.et_tm));
+            CHECK(rc);
+            CHECK(sec == tm2sec(&etm2.et_tm));
         }
     }
 }
 
 class my_path_source : public unique_path_source {
 public:
-    explicit my_path_source(ghc::filesystem::path p) : mps_path(std::move(p)) {
+    explicit my_path_source(ghc::filesystem::path p) : mps_path(std::move(p)) {}
 
-    }
-
-    ghc::filesystem::path get_path() const override {
+    ghc::filesystem::path get_path() const override
+    {
         return this->mps_path;
     }
 
     ghc::filesystem::path mps_path;
 };
 
-TEST_CASE("unique_path") {
+TEST_CASE("unique_path")
+{
     unique_path_generator upg;
 
     auto bar = make_shared<my_path_source>("/foo/bar");
@@ -179,3 +185,32 @@ TEST_CASE("unique_path") {
     CHECK(log1->get_unique_path() == "[machine1]/syslog.log");
     CHECK(log2->get_unique_path() == "[machine2]/syslog.log");
 }
+
+TEST_CASE("attr_line to json")
+{
+    attr_line_t al;
+
+    al.append("Hello, ").append(lnav::roles::symbol("World")).append("!");
+
+    auto json = lnav::to_json(al);
+    auto al2 = lnav::from_json<attr_line_t>(json).unwrap();
+    auto json2 = lnav::to_json(al2);
+
+    CHECK(json == json2);
+}
+
+TEST_CASE("user_message to json")
+{
+    auto um = lnav::console::user_message::error("testing")
+                  .with_reason("because")
+                  .with_snippet(lnav::console::snippet::from(
+                                    intern_string::lookup("hello.c"), "printf(")
+                                    .with_line(1))
+                  .with_help("close it");
+
+    auto json = lnav::to_json(um);
+    auto um2 = lnav::from_json<lnav::console::user_message>(json).unwrap();
+    auto json2 = lnav::to_json(um2);
+
+    CHECK(json == json2);
+}
