@@ -3,6 +3,40 @@
 echo ${top_srcdir}
 echo ${top_builddir}
 
+printf '#Date:\t20\x800-2-02\n0\n' | run_cap_test \
+    env TEST_COMMENT="short timestamp" ${lnav_test} -n
+
+printf '000\n000\n#Fields: 0\n0\n#Fields: 0\n0' | run_cap_test \
+    env TEST_COMMENT="invalid w3c log" ${lnav_test} -n
+
+cat > rollover_in.0 <<EOF
+2600/2 0 00:00:00 0:
+00:2 0 00:00:00 0:
+00:2 0 00:00:00 0:
+EOF
+touch -t 200711030923 rollover_in.0
+
+run_cap_test env TEST_COMMENT="invalid date rollover" ${lnav_test} -n rollover_in.0
+
+printf '#Fields: 0\tcs-bytes\n#Fields: 0\n\t0 #\n0' | run_cap_test \
+    env TEST_COMMENT="w3c with dupe #Fields" ${lnav_test} -n
+
+printf '#Fields: \xf9\t)\n0\n' | run_cap_test \
+    env TEST_COMMENT="garbage w3c fields #1" ${lnav_test} -n
+
+run_cap_test env TEST_COMMENT="w3c with bad header" ${lnav_test} -n <<EOF
+#Fields: 0 time
+ 00:00
+#Date:
+EOF
+
+printf '\x2b0\x1b[a' | run_cap_test \
+    env TEST_COMMENT="log line with an ansi escape" ${lnav_test} -n
+
+run_cap_test ${lnav_test} -n \
+    -c ';SELECT * FROM logline' \
+    ${test_dir}/logfile_block.1
+
 run_test ${lnav_test} -d /tmp/lnav.err -n -w logfile_stdin.0.log \
     -c ':shexec sleep 1 && touch -t 200711030923 logfile_stdin.0.log' <<EOF
 2013-06-06T19:13:20.123  Hi
@@ -12,6 +46,23 @@ check_output "piping to stdin is not working?" <<EOF
 2013-06-06T19:13:20.123  Hi
 EOF
 
+if test x"${TSHARK_CMD}" != x""; then
+  run_test env TZ=UTC ${lnav_test} -n ${test_dir}/dhcp.pcapng
+
+  check_output "pcap file is not recognized" <<EOF
+2004-12-05T19:16:24.317 0.0.0.0 → 255.255.255.255 DHCP 314 DHCP Discover - Transaction ID 0x3d1d
+2004-12-05T19:16:24.317 192.168.0.1 → 192.168.0.10 DHCP 342 DHCP Offer    - Transaction ID 0x3d1d
+2004-12-05T19:16:24.387 0.0.0.0 → 255.255.255.255 DHCP 314 DHCP Request  - Transaction ID 0x3d1e
+2004-12-05T19:16:24.387 192.168.0.1 → 192.168.0.10 DHCP 342 DHCP ACK      - Transaction ID 0x3d1e
+EOF
+
+  run_test ${lnav_test} -n ${test_dir}/dhcp-trunc.pcapng
+
+  check_error_output "truncated pcap file is not recognized" <<EOF
+error: unable to open file: {test_dir}/dhcp-trunc.pcapng -- tshark: The file "{test_dir}/dhcp-trunc.pcapng" appears to have been cut short in the middle of a packet.
+EOF
+fi
+
 
 cp ${srcdir}/logfile_syslog.0 truncfile.0
 chmod u+w truncfile.0
@@ -47,7 +98,13 @@ if test x"${LIBARCHIVE_LIBS}" != x""; then
       ${srcdir}/logfile_syslog.0
 
     check_error_output "invalid min-free-space allowed?" <<EOF
-command-option:1: error: value must be greater than or equal to 0, found -1
+✘ error: “-1” is not a valid value for option “/tuning/archive-manager/min-free-space”
+ reason: value must be greater than or equal to 0
+ --> input:1
+ = help: Property Synopsis
+           /tuning/archive-manager/min-free-space <bytes>
+         Description
+           The minimum free space, in bytes, to maintain when unpacking archives
 EOF
 
     rm -rf tmp/lnav-*
@@ -69,7 +126,8 @@ EOF
             `test_err_filename` > test_logfile.big.out
         mv test_logfile.big.out `test_err_filename`
         check_error_output "decompression worked?" <<EOF
-error: unable to open file: /logfile_syslog.1.xz -- available space on disk (NNN) is below the minimum-free threshold (1.0PB).  Unable to unpack 'logfile_syslog.1.xz' to 'tmp/lnav-user-NNN-work/archives/arc-NNN-logfile_syslog.1.xz'
+✘ error: unable to open file: /logfile_syslog.1.xz
+ reason: available space on disk (NNN) is below the minimum-free threshold (1.0PB).  Unable to unpack 'logfile_syslog.1.xz' to 'tmp/lnav-user-NNN-work/archives/arc-NNN-logfile_syslog.1.xz'
 EOF
 
         run_test env TMPDIR=tmp ${lnav_test} -n \
@@ -135,11 +193,12 @@ EOF
     run_test env TMPDIR=tmp ${lnav_test} -n \
         test-logs-trunc.tgz
 
-    sed -e "s|${builddir}||g" `test_err_filename` | head -1 \
+    sed -e "s|${builddir}||g" `test_err_filename` | head -2 \
         > test_logfile.trunc.out
     mv test_logfile.trunc.out `test_err_filename`
     check_error_output "truncated tgz not reported correctly" <<EOF
-error: unable to open file: /test-logs-trunc.tgz -- failed to read file: /test-logs-trunc.tgz >> src/lnav -- truncated gzip input
+✘ error: unable to open file: /test-logs-trunc.tgz
+ reason: failed to extract 'src/lnav' from archive '/test-logs-trunc.tgz' -- truncated gzip input
 EOF
 
     mkdir -p rotmp
@@ -147,13 +206,15 @@ EOF
     run_test env TMPDIR=rotmp ${lnav_test} -n test-logs.tgz
 
     sed -e "s|lnav-user-[0-9]*-work|lnav-user-NNN-work|g" \
+        -e 's|log\.0 -- .*|log\.0 -- ...|g' \
         -e "s|arc-[0-9a-z]*-test|arc-NNN-test|g" \
         -e "s|${builddir}||g" \
-        `test_err_filename` | head -1 \
+        `test_err_filename` | head -2 \
         > test_logfile.rotmp.out
     cp test_logfile.rotmp.out `test_err_filename`
     check_error_output "archive not unpacked" <<EOF
-error: unable to open file: /test-logs.tgz -- unable to write entry: rotmp/lnav-user-NNN-work/archives/arc-NNN-test-logs.tgz/test/logfile_access_log.0 -- Failed to create dir 'rotmp/lnav-user-NNN-work'
+✘ error: unable to open file: /test-logs.tgz
+ reason: unable to create directory: rotmp/lnav-user-NNN-work/archives -- Permission denied
 EOF
 fi
 
@@ -162,12 +223,13 @@ chmod ugo-r unreadable.log
 
 run_test ${lnav_test} -n unreadable.log
 
-sed -e "s|/.*/unreadable.log|unreadable.log|g" `test_err_filename` | head -1 \
+sed -e "s|/.*/unreadable.log|unreadable.log|g" `test_err_filename` | head -3 \
     > test_logfile.unreadable.out
 
 mv test_logfile.unreadable.out `test_err_filename`
 check_error_output "able to read an unreadable log file?" <<EOF
-Cannot read file: unreadable.log -- Permission denied
+✘ error: file exists, but is not readable: unreadable.log
+ reason: Permission denied
 EOF
 
 run_test ${lnav_test} -n 'unreadable.*'
@@ -306,6 +368,15 @@ Apr 06 10:30:11 2014 -- 474
 Apr 06 11:01:11 2014 -- 475
 EOF
 
+run_test ${lnav_test} -n ${srcdir}/logfile_tcf.1
+
+check_output "timestamps with no dates are not rewritten?" <<EOF
+TCF 2014-04-06 09:59:47.191000: Server-Properties: {"Name":"TCF Protocol Logger","OSName":"Linux 3.2.0-60-generic","UserName":"xavier","AgentID":"1fde3dd1-d4be-4f79-8090-6f8d212f03bf","TransportName":"TCP","Proxy":"","ValueAdd":"1","Port":"1534"}
+TCF 2014-04-06 10:30:11.474000: 0: ---> C 2 RunControl getChildren null <eom>
+TCF 2014-04-06 11:01:11.475000: 0: <--- R 2  ["P1"] <eom>
+EOF
+
+
 # The TCSH format converts to local time, so we need to specify a TZ
 export TZ="UTC"
 run_test ./drive_logfile -t -f tcsh_history ${srcdir}/logfile_tcsh_history.0
@@ -497,6 +568,26 @@ info 0x0
 error 0x0
 EOF
 
+run_test ./drive_logfile -t -f logfmt_log ${srcdir}/logfile_logfmt.0
+
+check_output "logfmt_log time interpreted incorrectly?" <<EOF
+Sep 15 21:17:10 2021 -- 220
+Sep 15 21:17:11 2021 -- 674
+Sep 15 21:17:11 2021 -- 678
+Sep 15 21:17:11 2021 -- 679
+Sep 15 21:18:20 2021 -- 335
+EOF
+
+run_test ./drive_logfile -v -f logfmt_log ${srcdir}/logfile_logfmt.0
+
+check_output "logfmt_log level interpreted incorrectly?" <<EOF
+error 0x0
+warning 0x0
+info 0x0
+info 0x0
+error 0x0
+EOF
+
 run_test ${lnav_test} -d /tmp/lnav.err -nt -w logfile_stdin.log <<EOF
 Hi
 EOF
@@ -511,7 +602,7 @@ run_test ${lnav_test} -C ${test_dir}/logfile_bad_access_log.0
 sed -ibak -e "s|/.*/logfile_bad_access_log.0|logfile_bad_access_log.0|g" `test_err_filename`
 
 check_error_output "bad access_log line not found?" <<EOF
-error:logfile_bad_access_log.0:1:line did not match format access_log/regex/std
+error:logfile_bad_access_log.0:1:line did not match format /access_log/regex/std
 error:logfile_bad_access_log.0:1:         line -- 192.168.202.254 [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkboot.gz HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
 error:logfile_bad_access_log.0:1:partial match -- 192.168.202.254
 EOF
